package internal

import (
	"crypto/md5"
	"encoding/hex"
	"gitea.cdlsxd.cn/BaseSystem/plugin/utils"
	"net/url"
	"plugins/zltx/internal/po"
	"strings"
)

func sign(values []byte) string {
	hash := md5.Sum(values)
	return strings.ToUpper(hex.EncodeToString(hash[:]))
}

func urlValues(req po.Req, appKey string) url.Values {
	uv := url.Values{}
	var kvStr []string
	kvRows := utils.SortStruct(req)
	for _, kv := range kvRows {
		if kv.Key == "sign" {
			continue
		}
		uv.Set(kv.Key, kv.Value)
		if kv.Key != "version" {
			kvStr = append(kvStr, kv.Key+"="+kv.Value)
		}
	}
	kvStr = append(kvStr, "key="+appKey)
	signStr := strings.Join(kvStr, "&")
	uv.Set("sign", sign([]byte(signStr)))

	return uv
}

func req(req po.Req, appKey string) (url.Values, error) {
	if err := req.Validate(); err != nil {
		return nil, err
	}
	return urlValues(req, appKey), nil
}

func verify(notify *po.Notify, appKey string) bool {
	kvRows := utils.SortStruct(notify)
	var kvStr []string

	for _, kv := range kvRows {
		if kv.Key != "sign" && kv.Key != "version" {
			kvStr = append(kvStr, kv.Key+"="+kv.Value)
		}
	}
	kvStr = append(kvStr, "key="+appKey)
	signStr := strings.Join(kvStr, "&")

	return sign([]byte(signStr)) == notify.Sign
}
